-- -----------------------------------------------------------------------------
--
--  Licensed Materials - Property of IBM
--  5698-B34  Copyright IBM Corp. 2008, 2011
--  All rights reserved.
--
--  US Government Users Restricted Rights - Use, duplication or
--  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
--  RESTRICTED RIGHTS:
--
--    This file may have been supplied under a license.
--    It may be used, disclosed, and/or copied only as permitted
--    under such license agreement.  Any copy must contain the
--    above copyright notice and this restricted rights notice.
--    Use, copying, and/or disclosure of the file is strictly
--    prohibited unless otherwise provided in the license agreement.
--
--  File: tivoli_eif/zos_files/upgrade/zOSEventSupport_sysplex.sql, OMNIbus, EventPump
--  SID: 1.3
--  Date Modified: 11/03/11 09:35:51

--
--  Desription:
--
--    This sql file is used to modify the alert.status table structure by adding
--    several columns and some automation trigger code to to the alerts.status
--    table required to support incoming z/OS events. This file is applied to
--    the Collection(primary) server in a typical 3-tier ESF configuration.
--
--  Change History:
--    YY/MM/DD            Description
--    --------            ------------------------------------------------------
--    10/01/14            $01,RAR: Add new column for SYSPLEX and compare for
--                        clear events
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
--  Add additional z/OS columns to the status table. These columns are used for
--  clearing events across the SYSPLEX.
-- -----------------------------------------------------------------------------
alter table alerts.status add column ZOS_Sysplex char(8);
go
alter table alerts.status add column ZOS_Smfid char(4);
go
alter table alerts.status add column ZOS_Sysname char(8);
go

-- -----------------------------------------------------------------------------
--  Update the zos_clear trigger to taken into account the SYSPLEX.
-- -----------------------------------------------------------------------------
create or replace trigger zos_clear
group zos_triggers
debug FALSE
priority 1
comment 'Aggregate server clearing for zos alerts only.'
after insert on alerts.status
for each row
when new.AlertGroup = 'TBSMV3_SOURCE390'
declare now utc;
begin

-- -----------------------------------------------------------------------------
--  If Message (all zos action code should colapse to '03' if the event
--  is a message).
-- -----------------------------------------------------------------------------
if( new.ZOS_Action = '03'  and new.Severity = 0 ) -- message
then
    -- --------------------------------------------------------------------7@01A
    --  Add SYSPLEX to event clearing logic.
    --  If the new event include a Sysplex value then use it in the clearing
    --  compare, but for older events it will compare on a Sysplex of null.
    --  If the new event does not include a Sysplex value then it will be
    --  ignored for the comparision.
    -- -------------------------------------------------------------------------

    -- -------------------------------------------------------------------------
    --  High level clear from a monitor sucah as SA/390 or OPS/MVS.
    -- -------------------------------------------------------------------------
    if( new.Node <> '' and new.ZOS_ResourceName <> '' and new.ZOS_SubResourceName = '' and new.ZOS_SubResourceType = '' )
    then
        if( new.ZOS_Sysplex <> '' )                                     -- 3@01A
        then
            update alerts.status set Severity = 0 where ZOS_Sysplex = new.ZOS_Sysplex and ZOS_ResourceName = new.ZOS_ResourceName
        else
            update alerts.status set Severity = 0 where Node = new.Node and ZOS_ResourceName = new.ZOS_ResourceName
        end if;
    end if;

    -- -------------------------------------------------------------------------
    --  Low level clear from a subtype resource.
    -- -------------------------------------------------------------------------
    if( new.Node <> '' and new.ZOS_ResourceName <>'' and new.ZOS_SubResourceName <> '' and new.ZOS_SubResourceType <> '' )
    then
        if( new.ZOS_Sysplex <> '' )                                     -- 3@01A
        then
            update alerts.status set Severity = 0 where ZOS_Sysplex = new.ZOS_Sysplex and ZOS_ResourceName = new.ZOS_ResourceName and  ZOS_SubResourceName = new.ZOS_SubResourceName and ZOS_SubResourceType= new.ZOS_SubResourceType
        else
            update alerts.status set Severity = 0 where Node = new.Node and ZOS_ResourceName = new.ZOS_ResourceName and  ZOS_SubResourceName = new.ZOS_SubResourceName and ZOS_SubResourceType= new.ZOS_SubResourceType
        end if;
    end if;

end if;

-- -----------------------------------------------------------------------------
--  If Resolution ( all zos action codes should colapse to '05' or
--  ('04' and 'OK') if it truely is a resolution alert).
-- -----------------------------------------------------------------------------
if( new.ZOS_Action = '05' or (new.ZOS_Action = '04' and new.Summary = 'OK') )
then
    -- -------------------------------------------------------------------------
    --  High level clear from a monitor such as SA/390 or OPS/MVS.
    -- -------------------------------------------------------------------------
    if( new.Node <> '' and new.ZOS_ResourceName <> '' and new.ZOS_SubResourceName = '' and new.ZOS_SubResourceType = '' )
    then
        if( new.ZOS_Sysplex <> '' )                                     -- 3@01A
        then
            update alerts.status set Severity = 0,Summary = 'Resolved: ' + Summary where ZOS_Sysplex = new.ZOS_Sysplex and ZOS_ResourceName = new.ZOS_ResourceName and ZOS_EventID = new.ZOS_EventID and Summary not like '^.*Resolved:.*'
        else
            update alerts.status set Severity = 0,Summary = 'Resolved: ' + Summary where Node = new.Node and ZOS_ResourceName = new.ZOS_ResourceName and ZOS_EventID = new.ZOS_EventID and Summary not like '^.*Resolved:.*'
        end if;
    end if;


    -- -------------------------------------------------------------------------
    --  Low level clear from a subtype resource monitor such as DB2. IMS or CICS
    -- -------------------------------------------------------------------------
    if( new.Node <> '' and new.ZOS_ResourceName <>'' and new.ZOS_SubResourceName <> '' and new.ZOS_SubResourceType <> '' )
    then
        if( new.ZOS_Sysplex <> '' )                                     -- 3@01A
        then
            update alerts.status set Severity = 0,Summary = 'Resolved: ' + Summary where ZOS_Sysplex = new.ZOS_Sysplex and ZOS_ResourceName = new.ZOS_ResourceName and  ZOS_SubResourceName = new.ZOS_SubResourceName and ZOS_SubResourceType= new.ZOS_SubResourceType and  ZOS_EventID = new.ZOS_EventID and Summary not like '^.*Resolved:.*'
        else
            update alerts.status set Severity = 0,Summary = 'Resolved: ' + Summary where Node = new.Node and ZOS_ResourceName = new.ZOS_ResourceName and  ZOS_SubResourceName = new.ZOS_SubResourceName and ZOS_SubResourceType= new.ZOS_SubResourceType and  ZOS_EventID = new.ZOS_EventID and Summary not like '^.*Resolved:.*'
        end if;
    end if;

end if;

end
go


create or replace trigger zos_deduplication
group zos_triggers
debug FALSE
priority 1
comment 'Aggregate server deduplication processing for zos alerts only.'
before reinsert on alerts.status
for each row
when old.AlertGroup = 'TBSMV3_SOURCE390'
declare
now utc;
begin

-- -----------------------------------------------------------------------------
--   Assign the z/OS severity value first.
-- -----------------------------------------------------------------------------
if(new.ZOS_IsUserSeverity=1)
then
    -- -------------------------------------------------------------------------
    --  Determine if event is a Message, Exception or Resolution
    --  Check value of action
    -- -------------------------------------------------------------------------

    -- -------------------------------------------------------------------------
    --  If Message (all zos action code should colapse to '03' it it truely
    --  is a message).
    -- -------------------------------------------------------------------------
    if( new.ZOS_Action = '03' ) -- message
    then
        for each row tbl2 in alerts.zos_severity_msgstate_lookup where tbl2.IsMessage = 1
            and new.ZOS_EventID like tbl2.EventID
        begin
            set old.Severity = tbl2.Severity;
            set old.ZOS_MsgState = tbl2.MsgState;
        end;
    end if;

    -- -------------------------------------------------------------------------
    --  If Exception (all zos action code should colapse to '04' it it truely
    --  is an exception alert).
    -- -------------------------------------------------------------------------
    if( new.ZOS_Action = '04' ) -- exception
    then
        -- ---------------------------------------------------------------------
        --  Lookup Severity based on EventID (using like statement)
        --  Insert the event with new Severity and State
        -- ---------------------------------------------------------------------
        for each row tbl in alerts.zos_severity_msgstate_lookup where  tbl.IsMessage = 0 and new.ZOS_EventID like tbl.EventID
        begin
            set old.Severity = tbl.Severity;
        end;
    end if;

end if;

-- -----------------------------------------------------------------------------
--  Do z/OS clering next
--  If Message (all zos action code should colapse to '03' if the event
--  is a message).
-- -----------------------------------------------------------------------------
if( new.ZOS_Action = '03'  and new.Severity = 0) -- message
then
    -- --------------------------------------------------------------------7@01A
    --  Add SYSPLEX to event clearing logic.
    --  If the new event include a Sysplex value then use it in the clearing
    --  compare, but for older events it will compare on a Sysplex of null.
    --  If the new event does not include a Sysplex value then it will be
    --  ignored for the comparision.
    -- -------------------------------------------------------------------------

    -- -------------------------------------------------------------------------
    --  High level clear from a monitor sucah as SA/390 or OPS/MVS.
    -- -------------------------------------------------------------------------
    if( new.Node <> '' and new.ZOS_ResourceName <> '' and new.ZOS_SubResourceName = '' and new.ZOS_SubResourceType = '' )
    then
        if( new.ZOS_Sysplex <> '' )                                     -- 3@01A
        then
            update alerts.status set Severity = 0 where ZOS_Sysplex = new.ZOS_Sysplex and ZOS_ResourceName = new.ZOS_ResourceName
        else
            update alerts.status set Severity = 0 where Node = new.Node and ZOS_ResourceName = new.ZOS_ResourceName
        end if;
    end if;

    -- -------------------------------------------------------------------------
    --  Low level clear from a subtype resource.
    -- -------------------------------------------------------------------------
    if( new.Node <> '' and new.ZOS_ResourceName <> '' and new.ZOS_SubResourceName <> '' and new.ZOS_SubResourceType <> '' )
    then
        if( new.ZOS_Sysplex <> '' )                                     -- 3@01A
        then
            update alerts.status set Severity = 0 where ZOS_Sysplex = new.ZOS_Sysplex and ZOS_ResourceName = new.ZOS_ResourceName and  ZOS_SubResourceName = new.ZOS_SubResourceName and ZOS_SubResourceType= new.ZOS_SubResourceType
        else
            update alerts.status set Severity = 0 where Node = new.Node and ZOS_ResourceName = new.ZOS_ResourceName and  ZOS_SubResourceName = new.ZOS_SubResourceName and ZOS_SubResourceType= new.ZOS_SubResourceType
        end if;
    end if;

end if;

-- -----------------------------------------------------------------------------
--  If Resolution (all zos action codes should colapse to '05'
--  or ('04' and 'OK') if it truely is a resolution alert).
-- -----------------------------------------------------------------------------
if( new.ZOS_Action = '05' or (new.ZOS_Action = '04' and new.Summary = 'OK') )
then
    -- -------------------------------------------------------------------------
    --  High level clear from a monitor such as SA/390 or OPS/MVS.
    -- -------------------------------------------------------------------------
    if( new.Node <> '' and new.ZOS_ResourceName <> '' and new.ZOS_SubResourceName = '' and new.ZOS_SubResourceType = '' )
    then
        if( new.ZOS_Sysplex <> '' )                                     -- 3@01A
        then
            update alerts.status set Severity = 0,Summary = 'Resolved: ' + Summary where ZOS_Sysplex = new.ZOS_Sysplex and ZOS_ResourceName = new.ZOS_ResourceName and ZOS_EventID = new.ZOS_EventID and Summary not like '^.*Resolved:.*'
        else
            update alerts.status set Severity = 0,Summary = 'Resolved: ' + Summary where Node = new.Node and ZOS_ResourceName = new.ZOS_ResourceName and ZOS_EventID = new.ZOS_EventID and Summary not like '^.*Resolved:.*'
        end if;
    end if;

    -- -------------------------------------------------------------------------
    --  Low level clear from a subtype resource monitor such as DB2. IMS or CICS
    -- -------------------------------------------------------------------------
    if( new.Node <> '' and new.ZOS_ResourceName <> '' and new.ZOS_SubResourceName <> '' and new.ZOS_SubResourceType <> '' )
    then
        if( new.ZOS_Sysplex <> '' )                                     -- 3@01A
        then
            update alerts.status set Severity = 0,Summary = 'Resolved: ' + Summary where ZOS_Sysplex = new.ZOS_Sysplex and ZOS_ResourceName = new.ZOS_ResourceName and  ZOS_SubResourceName = new.ZOS_SubResourceName and ZOS_SubResourceType= new.ZOS_SubResourceType and  ZOS_EventID = new.ZOS_EventID and Summary not like '^.*Resolved:.*'
        else
            update alerts.status set Severity = 0,Summary = 'Resolved: ' + Summary where Node = new.Node and ZOS_ResourceName = new.ZOS_ResourceName and  ZOS_SubResourceName = new.ZOS_SubResourceName and ZOS_SubResourceType= new.ZOS_SubResourceType and  ZOS_EventID = new.ZOS_EventID and Summary not like '^.*Resolved:.*'
        end if;
    end if;

end if;

-- Do base dedupe last...
-- set old.Tally = old.Tally + 1;                                        --@03C1
set old.LastOccurrence = new.LastOccurrence;
set old.StateChange = getdate();
set old.InternalLast = getdate();
set old.Summary = new.Summary;
set old.AlertKey = new.AlertKey;

end
go
